module Banks
  class Transaction
    attr_accessor :date, :description, :amount, :type, :category
    def to_s
      return date.to_s + " " + category.to_s + " " + Banks.amount_to_s(amount) + " " + type + " " + description
    end
  end

  class TransactionArray < Array
    def total
      total = 0
      each do |transaction|
        total = total + transaction.amount
      end
      Banks.amount_to_s(total)
    end

    def total_positive
      total = 0
      each do |transaction|
        total = total + transaction.amount unless transaction.amount < 0
      end
      Banks.amount_to_s(total)
    end

    def total_negative
      total = 0
      each do |transaction|
        total = total + transaction.amount unless transaction.amount > 0
      end
      Banks.amount_to_s(total)
    end

    # Sorts transactions by date, from newer to older.
    def sort_by_date
      sort! {|x, y| y.date <=> x.date}
    end

    # Sort by amount. Negative to positive.
    def sort_by_amount
      sort! {|x, y| x.amount <=> y.amount}
    end

    # Sort by category.
    def sort_by_category
      sort! {|x, y| x.category <=> y.category}
    end

    # Remove transactions considered transfers to own accounts.
    def remove_transfers
      result = TransactionArray.new
      each do |transaction|
        should_add = true

        # Currently unavailable.
        puts "Removing transaction. " + transaction.to_s unless should_add == true 
        if should_add then result << transaction end
      end
      return result
    end

    # Return transactions within a date range.
    def by_date (begin_date, end_date)
      result = TransactionArray.new

      # Filter all less than end date.
      temp_array = TransactionArray.new
      each do |transaction|
        if transaction.date <= end_date then temp_array << transaction end
      end

      # Filter all greater than begin date.
      temp_array.each do |transaction|
        if transaction.date >= begin_date then result << transaction end
      end

      return result
    end

    # Return current month transactions.
    def current_month
      return previous_month 0
    end

    # Return last month transactions.
    def last_month
      return previous_month 1
    end

    # months_back - how many months to go back from current date.
    def previous_month(months_back)
      year = Date.today.year
      month = Date.today.month

      begin_date = Date.civil(year, month, 1) - months_back.month
      end_date = begin_date.end_of_month
      
      return by_date(begin_date, end_date)
    end

    def to_s
      result = ""
      each do |transaction|
        result = result + transaction.to_s + "\n"
      end
      result
    end
    
    def to_s_by_cat
      result = ""
      current_cat = nil
      total = 0
      last_transaction = nil
      each do |transaction|
        if !current_cat.nil? && current_cat != transaction.category
          result = result + "Total for category #{current_cat}: " + Banks.amount_to_s(total) + "\n\n"
          total = 0
        end
        result = result + transaction.to_s + "\n"
        total = total + transaction.amount
        current_cat = transaction.category
        last_transaction = transaction
      end
      if !last_transaction.nil? then
        result = result + "Total for category #{current_cat}: " + Banks.amount_to_s(total) + "\n\n"
        total = 0
      end
      result
    end

    # Merge array
    def merge_array(merging_array)
      merging_array.each do |transaction|
        self << transaction
      end
    end
    
    def << (transaction)
      super
      #transaction.category = (CategoryPolicy.categorize transaction.description).upcase
    end
  end
end
